/***********************************************************************************************************************
 * Copyright [2020-2022] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
 * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
 * sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for the selection and use
 * of Renesas products and Renesas assumes no liability.  No license, express or implied, to any intellectual property
 * right is granted by Renesas. This software is protected under all applicable laws, including copyright laws. Renesas
 * reserves the right to change or discontinue this software and/or this documentation. THE SOFTWARE AND DOCUMENTATION
 * IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND TO THE FULLEST EXTENT
 * PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY, INCLUDING WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE SOFTWARE OR
 * DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.  TO THE MAXIMUM
 * EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR DOCUMENTATION
 * (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER, INCLUDING,
 * WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY LOST PROFITS,
 * OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/

#include "hal_entry.h"
#include "rm_icm42670/rm_icm42670.h"

#include "models/motionDetect_BC1_v12/motionDetect_BC1_v12_model.h"

#define RM_ICM42670_EXAMPLE_DELAY_50MS 50
#define RM_ICM42670_EXAMPLE_DELAY_1US 10
#define RM_ICM42670_EXAMPLE_IRQ_ENABLE 1

void R_BSP_WarmStart(bsp_warm_start_event_t event);
fsp_err_t rm_icm42670_irq_open (rm_icm42670_ctrl_t * const p_api_ctrl);

bool g_i2c_flag = 0;
bool g_irq_flag = 0;

const char *Console_Terminator_String = "\r\n";
const char *Display_Terminator_String = "";

#define  NUM_BUFFS 2
#define WINDOW_SIZE (512)
#define BUFF_LEN (WINDOW_SIZE)
#define CFG_SYNC_WORD       (0x53534E52) // 'RNSS' backwards
#define BUFFER_LENGTH 100
#define NUM_K_CLASSES 5
#define NUM_S_CLASSES 5
#define SMOOTH_LEN 5

static int count = 0;
float data_x[BUFF_LEN];
float data_y[BUFF_LEN];
float data_z[BUFF_LEN];

static uint32_t g_events = 0;
static uint32_t g_err_count = 0;
static bool g_dc_callback = false;
static rai_data_collector_callback_args_t g_callback_args;
rai_data_collector_frame_buffer_t g_frame_buf[8];

void rai_data_collector0_callback(const rai_data_collector_callback_args_t *p_args)
{
    g_callback_args.frame_buf_len = p_args->frame_buf_len;
    g_callback_args.frames = p_args->frames;
    g_callback_args.instance_id = p_args->instance_id;
    for(uint8_t i = 0; i < p_args->frames; i++)
    {
        g_frame_buf[i] = p_args->p_frame_buf[i];
    }
    g_callback_args.p_frame_buf = g_frame_buf;

    g_dc_callback = true;
}

void rai_data_collector0_error_callback(const rai_data_collector_error_callback_args_t *p_args)
{
    g_events |= p_args->event;
}

void rai_data_collector_error_callback(const rai_data_collector_error_callback_args_t *p_args)
{
    g_events |= p_args->event;
}

void rai_data_shipper0_callback(rai_data_shipper_callback_args_t *p_args)
{
    if(p_args->result != FSP_SUCCESS)
    {
        g_err_count++;
    }
    g_rai_data_collector0.p_api->bufferRelease(&g_rai_data_collector0_ctrl);
}


/*
 * For Motion detection functions
 */
typedef struct predictions_buff
{
    uint32_t sync_word;
    // declaring the enum for model prediction class
    enum  motionDetect_BC1_v12_AICLASS pred_class;
}prediction_buff;

float input_data[3*BUFF_LEN]; // will contain the raw data instead of interleave data


#define SOFTWARE_DELAY (100) //100 ms
#define GPT_EXAMPLE_MAX_PERCENT                   (100)
/* This example shows how to calculate a new duty cycle value at runtime. */
void gpt_duty_cycle_calculation (timer_ctrl_t * const timer_ctrl, uint32_t const pin, int duty_cycle_percent)
{
    fsp_err_t err = FSP_SUCCESS;

    /*
     *  So we set the period of 0x1000, or 65536 counts of the PCLKD source.
     *  In the “Clocks” tab you can see that it is 48 MHz,
     *  so the timer frequency is 48 / 65536 = 732 Hz, and the timer period is 1000000 / 732 = 1365 us.
     */

    /* Get the current period setting. */
    timer_info_t info;
    (void) R_GPT_InfoGet(timer_ctrl, &info);
    uint32_t current_period_counts = info.period_counts;
    /* Calculate the desired duty cycle based on the current period. Note that if the period could be larger than
     * UINT32_MAX / 100, this calculation could overflow. A cast to uint64_t is used to prevent this. The cast is
     * not required for 16-bit timers. */
    uint32_t duty_cycle_counts =
        (uint32_t) (((uint64_t) current_period_counts * duty_cycle_percent) /
                    GPT_EXAMPLE_MAX_PERCENT);
    /* Set the calculated duty cycle. */
    err = R_GPT_DutyCycleSet(timer_ctrl, duty_cycle_counts, pin);
    assert(FSP_SUCCESS == err);
}


void clearLeds(void){
    // red led
    R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_06_PIN_10, 0);

    //green led
    R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_06_PIN_09, 0);


    int red_duty_cycle = 0;
    int green_duty_cycle = 0;
    int blue_duty_cycle = 0;

    // RGB green BSP_IO_PORT_06_PIN_03 -> GPT_IO_PIN_GTIOCA
    gpt_duty_cycle_calculation(&g_timer1_ctrl,GPT_IO_PIN_GTIOCA,100-green_duty_cycle);

    // RGB red - BSP_IO_PORT_06_PIN_02 -> GPT_IO_PIN_GTIOCB
    gpt_duty_cycle_calculation(&g_timer1_ctrl,GPT_IO_PIN_GTIOCB,100-red_duty_cycle);

    // RGB blue - BSP_IO_PORT_06_PIN_05 -> GPT_IO_PIN_GTIOCA
    gpt_duty_cycle_calculation(&g_timer2_ctrl, GPT_IO_PIN_GTIOCA,100);
}


void blinkDropLed(void){
    clearLeds();
    int count = 0;
    while(count<5)// 1000 msec
    {
        R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_06_PIN_10, 1);
        R_BSP_SoftwareDelay(SOFTWARE_DELAY, BSP_DELAY_UNITS_MILLISECONDS);
        R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_06_PIN_10, 0);
        R_BSP_SoftwareDelay(SOFTWARE_DELAY, BSP_DELAY_UNITS_MILLISECONDS);
        count++;
    }
}

void RGBLed(int blue_duty_cycle, int red_duty_cycle, int green_duty_cycle){

    // RGB green BSP_IO_PORT_06_PIN_03 -> GPT_IO_PIN_GTIOCA
    gpt_duty_cycle_calculation(&g_timer1_ctrl,GPT_IO_PIN_GTIOCA,100-green_duty_cycle);

    // RGB red - BSP_IO_PORT_06_PIN_02 -> GPT_IO_PIN_GTIOCB
    gpt_duty_cycle_calculation(&g_timer1_ctrl,GPT_IO_PIN_GTIOCB,100-red_duty_cycle);

    // RGB blue - BSP_IO_PORT_06_PIN_05 -> GPT_IO_PIN_GTIOCA
    gpt_duty_cycle_calculation(&g_timer2_ctrl, GPT_IO_PIN_GTIOCA, 100-blue_duty_cycle);
}

void inMotionLed(void){
    // magenta color onRGB LED
    clearLeds();
    int blue_duty_cycle = 20;
    int red_duty_cycle = 100-blue_duty_cycle;
    int green_duty_cycle = 0;
    RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);

//    R_BSP_SoftwareDelay(, BSP_DELAY_UNITS_MILLISECONDS);
}

void waveLed(void){
    //yellow color on RGB LED
    clearLeds();
//    int blue_duty_cycle = 0;
//    int red_duty_cycle = 80;
//    int green_duty_cycle = 100-red_duty_cycle;
//    RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
    int red_duty_cycle = 100;
    int green_duty_cycle = 0;
    int blue_duty_cycle = 0;
    int count = 0;
    int MS_PER_COLOR = 50;
    int COLORS = 3;
    int sec = 1;
    while (count<(sec*1000/(COLORS*MS_PER_COLOR)))
    {
        red_duty_cycle = 100;
        green_duty_cycle = 0;
        blue_duty_cycle = 0;

        RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
        R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);

        red_duty_cycle = 0;
        green_duty_cycle = 100;
        blue_duty_cycle = 0;
        RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
        R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);

        red_duty_cycle = 0;
        green_duty_cycle = 0;
        blue_duty_cycle = 100;
        RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
        R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);
            count++;
    }
    clearLeds();
}

void circleLed(void){
    //blue color blink on RGB LED
    clearLeds();
    int red_duty_cycle = 100;
    int green_duty_cycle = 0;
    int blue_duty_cycle = 0;
    int count = 0;
    int MS_PER_COLOR = 50;
    int COLORS = 2;
    int sec = 1;
//    while (count<(sec*1000/(COLORS*MS_PER_COLOR)))
//    {
        red_duty_cycle = 0;
        green_duty_cycle = 0;
        blue_duty_cycle = 100;
        RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
        R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);

//        red_duty_cycle = 0;
//        green_duty_cycle = 0;
//        blue_duty_cycle = 0;
//        RGBLed(blue_duty_cycle, red_duty_cycle, green_duty_cycle);
//        R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);
//        count++;
//    }
//    clearLeds();
}

void idleLed(void){
    clearLeds();
    R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_06_PIN_09, 1);
}



/*******************************************************************************************************************//**
 * @brief  Icm42670 IRQ mode example application
 *
 * Open Icm42670, read Accel, Gyro, Temperature data in IRQ mode, and send the data with uart.
 *
 **********************************************************************************************************************/
void hal_entry (void)
{

#if BSP_TZ_SECURE_BUILD

    /* Enter non-secure code */
    R_BSP_NonSecureEnter();
#endif

    uint32_t test_pin = 0;
    fsp_err_t              err = FSP_SUCCESS;
    rm_icm42670_raw_data_t   raw_data;
    rm_icm42670_accel_data_t icm42670_accel_data;
    rm_icm42670_gyro_data_t icm42670_gyro_data;
    rm_icm42670_temp_data_t  icm42670_temp_data;
#if 0 == RM_ICM42670_EXAMPLE_IRQ_ENABLE
    rm_icm42670_device_status_t device_status;
#endif
    
    /* Open the uart bus if it is not already open. */
    //rm_uart_initialize();

    /* Enable access to the PFS registers. If using r_ioport module then register protection is automatically
     * handled. This code uses BSP IO functions to show how it is used.
     */
     R_BSP_PinAccessEnable();

    /* Open the I2C bus if it is not already open. */
    rm_comms_i2c_bus_extended_cfg_t * p_extend =
        (rm_comms_i2c_bus_extended_cfg_t *) g_icm42670_sensor0_cfg.p_comms_instance->p_cfg->p_extend;
    i2c_master_instance_t * p_driver_instance = (i2c_master_instance_t *) p_extend->p_driver_instance;
    p_driver_instance->p_ctrl = &g_icm42670_sensor0_ctrl;
    p_driver_instance->p_api->open(p_driver_instance->p_ctrl, p_driver_instance->p_cfg);
#if BSP_CFG_RTOS
    /* Create a semaphore for blocking if a semaphore is not NULL */
    if (NULL != p_extend->p_blocking_semaphore)
    {
 #if BSP_CFG_RTOS == 1                 // AzureOS
        tx_semaphore_create(p_extend->p_blocking_semaphore->p_semaphore_handle,
                            p_extend->p_blocking_semaphore->p_semaphore_name,
                            (ULONG) 0);
 #elif BSP_CFG_RTOS == 2               // FreeRTOS
        *(p_extend->p_blocking_semaphore->p_semaphore_handle) =
            xSemaphoreCreatecountingStatic((UBaseType_t) 1,
                                           (UBaseType_t) 0,
                                           p_extend->p_blocking_semaphore->p_semaphore_memory);
 #endif
    }
    /* Create a recursive mutex for bus lock if a recursive mutex is not NULL */
    if (NULL != p_extend->p_bus_recursive_mutex)
    {
 #if BSP_CFG_RTOS == 1                 // AzureOS
        tx_mutex_create(p_extend->p_bus_recursive_mutex->p_mutex_handle,
                        p_extend->p_bus_recursive_mutex->p_mutex_name,
                        TX_INHERIT);
 #elif BSP_CFG_RTOS == 2               // FreeRTOS
        *(p_extend->p_bus_recursive_mutex->p_mutex_handle) =
            xSemaphoreCreateRecursiveMutexStatic(p_extend->p_bus_recursive_mutex->p_mutex_memory);
 #endif
    }
#endif

    /* Open ICM42670 */
    err = RM_ICM42670_Open(&g_icm42670_sensor0_ctrl, &g_icm42670_sensor0_cfg);
    if(err != FSP_SUCCESS){__BKPT(0);}

    err = RM_ICM42670_DeviceInterruptCfgSet(&g_icm42670_sensor0_ctrl, g_icm42670_interrupt_cfg);
    if(err != FSP_SUCCESS){__BKPT(0);}

    /* Start measurement in data ready mode */
    err = RM_ICM42670_MeasurementStart(&g_icm42670_sensor0_ctrl);
    if(err != FSP_SUCCESS){__BKPT(0);}


    /* Open external IRQ */
    rm_icm42670_irq_open(&g_icm42670_sensor0_ctrl);

    R_ICU_ExternalIrqEnable(&g_external_irq1_ctrl);

    err = RM_RAI_DATA_COLLECTOR_Open(&g_rai_data_collector0_ctrl, &g_rai_data_collector0_cfg);
    err = RM_RAI_DATA_SHIPPER_Open(&g_rai_data_shipper0_ctrl, &g_rai_data_shipper0_cfg);

    err = RM_RAI_DATA_COLLECTOR_ChannelBufferGet(&g_rai_data_collector0_ctrl, 0, (void*)data_x);
    err = RM_RAI_DATA_COLLECTOR_ChannelBufferGet(&g_rai_data_collector0_ctrl, 1, (void*)data_y);
    err = RM_RAI_DATA_COLLECTOR_ChannelBufferGet(&g_rai_data_collector0_ctrl, 2, (void*)data_z);

    /*
     * Initialize the prediction struct
     */
    prediction_buff pred_data;

    /* Initializes the timer modules for LEDs */
    fsp_err_t status = R_GPT_Open(&g_timer1_ctrl, &g_timer1_cfg);
    /* Handle any errors. This function should be defined by the user. */
    assert(FSP_SUCCESS ==status);
    /* Start the timer. */
    (void) R_GPT_Start(&g_timer1_ctrl);

    status = R_GPT_Open(&g_timer2_ctrl, &g_timer2_cfg);
    /* Handle any errors. This function should be defined by the user. */
    assert(FSP_SUCCESS ==status);
    /* Start the timer. */
    (void) R_GPT_Start(&g_timer2_ctrl);


    clearLeds();
    pred_data.pred_class = -1;

    while (true)
    {
#if RM_ICM42670_EXAMPLE_IRQ_ENABLE

        /* Wait IRQ callback */
        while (0 == g_irq_flag)
        {
            /* Wait callback */
        }
        g_irq_flag = 0;
#else
        do
        {
            RM_ICM42670_DeviceStatusGet(&g_icm42670_sensor0_ctrl, &device_status);
            rm_icm42670_device_status_check(&g_icm42670_sensor0_ctrl);
        } while (false == device_status.data_ready);
#endif
#if 1

        /* Read Accel data */
        RM_ICM42670_AccelRead(&g_icm42670_sensor0_ctrl, &raw_data);
        /* Calculate Accel data */
        RM_ICM42670_AccelDataCalculate(&g_icm42670_sensor0_ctrl, &raw_data, &icm42670_accel_data);

        /* Continue to accumulate samples. */
        if(count < BUFF_LEN){
            //accel_data.data[count] = sqrt((icm42670_accel_data.accel_x*icm42670_accel_data.accel_x) +
            //                              (icm42670_accel_data.accel_y*icm42670_accel_data.accel_y) +
            //                              (icm42670_accel_data.accel_z*icm42670_accel_data.accel_z));
            data_x[count] = icm42670_accel_data.accel_x;
            data_y[count] = icm42670_accel_data.accel_y;
            data_z[count] = icm42670_accel_data.accel_z;
            input_data[count] = icm42670_accel_data.accel_x;
            input_data[count+WINDOW_SIZE] = icm42670_accel_data.accel_y;
            input_data[count+2*WINDOW_SIZE] = icm42670_accel_data.accel_z;
            count++;
        }
        else{

            err = RM_RAI_DATA_COLLECTOR_ChannelWrite(&g_rai_data_collector0_ctrl, 0, data_x, BUFF_LEN);
            err = RM_RAI_DATA_COLLECTOR_ChannelWrite(&g_rai_data_collector0_ctrl, 1, data_y, BUFF_LEN);
            err = RM_RAI_DATA_COLLECTOR_ChannelWrite(&g_rai_data_collector0_ctrl, 2, data_z, BUFF_LEN);

            rai_data_shipper_write_params_t arg;
            arg.diagnostic_data_len = 0;
            arg.events = g_events;
            arg.p_diagnostic_data = NULL;
            arg.p_sensor_data = &g_callback_args;
            g_dc_callback = false;

            err = RM_RAI_DATA_SHIPPER_Write(&g_rai_data_shipper0_ctrl, &arg);
            g_dc_callback = false;

            pred_data.pred_class = motionDetect_BC1_v12_classifier_predictionfloat(input_data);
            //            err = R_SCI_UART_Write(&g_uart5_ctrl, (void*)&pred_data, sizeof(pred_data));

            switch (pred_data.pred_class){
                  case motionDetect_BC1_v12_drop: //motionDetect_E2C1_engSeg_v2_drop :
                      blinkDropLed();
                      break;
                  case motionDetect_BC1_v12_in_motion: //motionDetect_E2C1_engSeg_v2_hello :
                      inMotionLed();
                      break;
                  case motionDetect_BC1_v12_wave: //motionDetect_E2C1_engSeg_v2_wave :
                      waveLed();
                      break;
                  case motionDetect_BC1_v12_idle: //motionDetect_E2C1_engSeg_v2_idle :
                      idleLed();
                      break;
                  case motionDetect_BC1_v12_circle: //motionDetect_E2C1_engSeg_v2_idle :
                      circleLed();
                      break;
                  default:
                      clearLeds();
              }

              count = 0;
//            count -= (int)(WINDOW_SIZE/2) ;
        }


#endif
    }
}

/*******************************************************************************************************************//**
 * This function is called at various points during the startup process.  This implementation uses the event that is
 * called right before main() to set up the pins.
 *
 * @param[in]  event    Where at in the start up process the code is currently at
 **********************************************************************************************************************/
void R_BSP_WarmStart (bsp_warm_start_event_t event)
{
    if (BSP_WARM_START_RESET == event)
    {
#if BSP_FEATURE_FLASH_LP_VERSION != 0

        /* Enable reading from data flash. */
        R_FACI_LP->DFLCTL = 1U;

        /* Would normally have to wait tDSTOP(6us) for data flash recovery. Placing the enable here, before clock and
         * C runtime initialization, should negate the need for a delay since the initialization will typically take more than 6us. */
#endif
    }

    if (BSP_WARM_START_POST_C == event)
    {
        /* C runtime environment and system clocks are setup. */

        /* Configure pins. */
        R_IOPORT_Open(&g_ioport_ctrl, g_ioport.p_cfg);
    }
}
